// Summe.java                                       MM 2013
// Numerische Fehler durch Datendarstellung
// Optimierung der Rechenzeit

import Tools.IO.*;                              // Eingaben

/**
 * Programm berechnet die Summe der ersten
 * n natuerlichen Zahlen mittels verschiedener
 * Verfahren.
 * @author Monika Meiler
 */
public class Summe
{
/* ----------------------------------------------------- */
                                        // service-Methoden
/**
 * Berechnen der Summe von 0 bis n.
 * @param n letzter Summand
 * @return Summe 0 bis n (0 fuer n < 1)
 */
  public long summe( long n)
  {
    long summe = 0;
    for( long i = 1; i < n + 1; i++)
    {
      summe += i;
    }
    return summe;
  }

/**
 * Gauss-Formel, berechnet Summe von 0 bis n.
 * @param n letzter Summand
 * @return Summe 0 bis n (0 fuer n < 1)
 */
  public long gauss( long n)
  {
    long summe = 0;

    if( n % 2 == 0) summe = n / 2 * ( n + 1);
    else summe = ( n + 1) / 2 * n;

/*
    summe = (long)(n / 2. * ( n + 1));
*/

    return summe;
  }

/* ----------------------------------------------------- */

/**
 * Startet das Programm,
 * erzeugt Summe bis zu einer eingelesenen Zahl.
 */
  public static void main( String[] args)
  {

// Eingaben von n
//    long n = IOTools.readLong( "Summe von 0 bis ");

// n im kritischen Bereich
    long n = (long)Math.pow( 2.0, 32.0) - 1;
//    long n = (long)Math.pow( 2.0, 32.0);

    System.out.println( "Summe von 0 bis " + n);
    System.out.println
    ( "Berechnung kann einige Sekunden dauern!");

// Objekterzeugung
    Summe s = new Summe();

// Summenberechnung und Ausgabe
    System.out.println
    ( "Schleife : " + s.summe( n));

    System.out.println
    ( "Gauss    : " + s.gauss( n));

// Fertig
    System.out.println( "Programm beendet");
 }
}

/* ----------------------------------------------------- */
// Testergebnisse
/*
 Summe von 0 bis 4294967295 (2^32 - 1)
 Schleife : 9223372034707292160
 Gauss    : 9223372034707292160
            ->  korrekt

 Summe von 0 bis 4294967296 (2^32)
 Schleife : -9223372034707292160
 Gauss    : 9223372036854775807
            ->  FEHLER: Long.MAX_VALUE ueberschritten
 (n = 4294967296, s = 9223372039002259456)

 Summe mit double:
 Summe von 0 bis 1073741824 (2^30)
 double : 5.764607528402944E17
            ->  korrekt

 Summe von 0 bis 1073741825 (2^30 + 1)
 double : 5.7646075391403622E17
            ->  FEHLER: max. 16 gueltige Stellen
*/